#Function to calculate Structural Generalized Forecast Error Variance Decompositions
fevd <- function(shock="US.Dp",maxlag=1,shrinkage=0,F=gvarobj$F,G=gvarobj$G,x=gvarobj$x,Rot=rotpick,horizon=40,sigs1=gvarobj$sigs){
  covlist <- NULL
  K <- nrow(gvarobj$F[[1]])
  names <- substr(rownames(x),1,2)
  names <- names[!duplicated(names)]
  N <- length(names)
  resids <- G%*%err
  rownames(resids) <- rownames(err) <- rownames(x)
  #creates a block diagonal global VC matrix
  for (i in 1:N){
    if (names[[i]]=="US"){
      covlist[[i]] <- t(chol(sigs1[[i]]))
    }else{
      covlist[[i]] <- sigs1[[i]]
    }
  }
  
  gcov <- bdiag(covlist) #block diagonal global covariance matrix
  Cm <- as.matrix(gcov)
  colnames(gcov) <- rownames(x); rownames(gcov) <- rownames(x)
  
  lF <- array(0,c(K,K,maxlag))
  
  for (kk in 1:length(F)){
    lF[,,kk] <- F[[kk]]
  }
  
  PHIx <- array(0,c(K,K,maxlag+horizon+1));rownames(PHIx) <- colnames(PHIx) <- rownames(x);
  
  for (i in 1:maxlag){
    PHIx[,,i]  <-  matrix(0,K,K) #ok
  }
  PHIx[,,maxlag+1]  <-  diag(K)
  
  for (t in (maxlag+2):(maxlag+horizon+1)){
    acc  <-  0
    for (j in 1:maxlag){
      acc  <-  acc + lF[,,j]%*%PHIx[,,t-j]
    }
    PHIx[,,t]  <-  acc
  }
  
  #reindicize
  PHI  <-  PHIx[,,(maxlag+1):(maxlag+horizon+1)]
  
  
  GFEVDres  <-  matrix(0,K,horizon+1)
  vslct  <-  diag(K)
  
  eslct <-matrix(0,K,1);rownames(eslct) <- rownames(x) #input x
  eslct[shock,1] <- 1
  
  invG <-  diag(nrow(G))%*%solve(G) # G INPUT
  invGSigmau  <-  solve(G)%*%Cm
  
  num  <-  matrix(0,K,horizon+1)
  den  <-  matrix(0,K,horizon+1)
  
  scale  <-  1/diag(Cm)
  
  n <- 1
  while (n<=horizon+1){
    for (l in 1:n){
      acc1  <-  t((t(eslct)%*%Rot%*%PHI[,,l]%*%invGSigmau%*%vslct)^2)
      num[,n]  <-  num[,n] + acc1
      acc2  <-  (t(eslct)%*%Rot%*%PHI[,,l]%*%invGSigmau%*%t(invG)%*%t(Rot%*%PHI[,,l])%*%eslct)
      den[,n]  <-  den[,n] + matrix(1,K,1)*as.numeric(acc2)
    }
    GFEVDres[,n]  <-  (scale*num[,n])/den[,n]
    n <- n+1
  }
  return(FEVD=GFEVDres)
}